import java.io.IOException;
import java.io.OutputStream;
import org.apache.log4j.Level;
import org.jetel.component.AbstractGenericTransform;
import org.jetel.data.DataField;
import org.jetel.data.DataRecord;
import org.jetel.exception.ComponentNotReadyException;
import org.jetel.exception.ConfigurationStatus;
import org.jetel.exception.ConfigurationStatus.Priority;
import org.jetel.exception.ConfigurationStatus.Severity;
import org.jetel.exception.JetelRuntimeException;
import org.jetel.graph.dictionary.Dictionary;
import org.jetel.metadata.DataRecordMetadata;

import com.datastax.driver.core.Cluster;
import com.datastax.driver.core.DataType;
import com.datastax.driver.core.Host;
import com.datastax.driver.core.Metadata;
import com.datastax.driver.core.Session;
import com.datastax.driver.core.PreparedStatement;
import com.datastax.driver.core.BoundStatement;

/**
 * This is an example custom writer. It shows how you can
 *  read records from input port and write to dictionary.
 */
public class myCassandraWriter extends AbstractGenericTransform {

	@Override
	public void execute() {
		// get component properities
		String node = getProperties().getStringProperty("cassandraNode");
		String keySapce = getProperties().getStringProperty("cassandraKeySpace");
		String sql = getProperties().getStringProperty("cassandraSQL");
		
		// open connection to Cassandra
		Cluster cluster = Cluster.builder().addContactPoint(node).build();
		Session session = cluster.connect(keySapce);
		
		// compile batch statement
		PreparedStatement ps = session.prepare(sql);
		BoundStatement bs = ps.bind();
		
		// initialize input record
		DataRecord inputRecord;
		
		// write records to Cassandra
		while ((inputRecord = readRecordFromPort(0)) != null) {
			int fieldOffset = 0;
			String fieldType;
			String fieldName;
			
			DataField[] fields = inputRecord.getFields();
			
			for (DataField field: fields) {
				fieldType = field.getMetadata().getDataType().getName();
				fieldName = field.getMetadata().getName();
				
				switch (fieldType) {
					case "integer":
						bs.setInt(fieldOffset, (int)field.getValue());
						break;
					case "string":
						if (field.isNull()) {
							bs.setString(fieldOffset, "");
						}
						else {
							bs.setString(fieldOffset, field.getValue().toString());
						}
						break;
					default:
						throw new JetelRuntimeException("ERROR: Unknown data type \"" + fieldType + "\" of field \"" + fieldName + "\".");
				}
				
				fieldOffset++;
			}
			
			session.execute(bs);
		}
		
		// close connection to cassandra
		cluster.close();
		
		
		
		/** This code is an example. Replace it with your custom code. */

		/** This is how you can read component properties. */
		//String customValue = getProperties().getStringProperty("myCustomPropertyName");

		/** You can log messages. */
		//getLogger().log(Level.DEBUG, "Custom property resolved to: " + customValue);

		/** The target you want to write to. This could be a local file, a database table...
			In this example we write into graph's dictionary. */
		//Dictionary dictionary = getGraph().getDictionary();

		/** Record we use for reading from input port 0. */
		//DataRecord record;

		/** We are going to compute sum of numbers in input records. */
		//int sum = 0;

		/** Read all records from input port 0 and compute sum of the data. */
		/*
		while ((record = readRecordFromPort(0)) != null) {
			Object valueFromRecord = record.getField("myMetadataFieldName").getValue();
			sum += (Integer) valueFromRecord;
		}
		*/

		/** Write into graph's dictionary. */
		/*
		try {
			dictionary.setValue("myDictionaryEntry", sum);
		} catch (ComponentNotReadyException e) {
			throw new JetelRuntimeException(e);
		}
		*/

		/** Advanced example: Reading all input records and writing them to a file */
		/*
		try (OutputStream out = getOutputStream("data-out/file.txt", false)) {
			while ((record = readRecordFromPort(0)) != null) {
				String valueFromRecord = record.getField("field1").getValue().toString();
				byte[] bytes = (valueFromRecord + System.lineSeparator()).getBytes("UTF-8");
				out.write(bytes);
			}
		} catch (IOException e) {
			throw new JetelRuntimeException("Problem with writing output file", e);
		}
		*/
	}

	@Override
	public ConfigurationStatus checkConfig(ConfigurationStatus status) {
		super.checkConfig(status);

		/** This way you can check connected edges and their metadata. */
		/*
		if (getComponent().getInPorts().size() < 1) {
			status.add("Input port must be connected!", Severity.ERROR, getComponent(), Priority.NORMAL);
			return status;
		}

		DataRecordMetadata inMetadata = getComponent().getInputPort(0).getMetadata();
		if (inMetadata == null) {
			status.add("Metadata on input port not specified!", Severity.ERROR, getComponent(), Priority.NORMAL);
			return status;
		}

		if (inMetadata.getFieldPosition("myMetadataFieldName") == -1) {
			status.add("Incompatible input metadata!", Severity.ERROR, getComponent(), Priority.NORMAL);
		}
		*/
		return status;
	}

	@Override
	public void init() {
		super.init();
	}

	@Override
	public void preExecute() throws ComponentNotReadyException {
		super.preExecute();
	}

	@Override
	public void postExecute() throws ComponentNotReadyException {
		super.postExecute();
	}
}
